#! /usr/bin/env python

import sys, gzip, subprocess, re, time 
import pysam

def merge_SJ2(SJ_file_list, output_file, control_file, junc_num_thres, is_keep_annotated):

    # get junctions registered in control
    control_db = {}
    if control_file is not None:
        with gzip.open(control_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                key = F[0] + '\t' + F[1] + '\t' + F[2]
                control_db[key] = 1


    # list up junctions to pick up
    junc2list = {}
    for SJ_file in SJ_file_list:
        with open(SJ_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                if is_keep_annotated == False and F[5] != "0": continue
                if int(F[6]) < junc_num_thres: continue

                key = F[0] + '\t' + F[1] + '\t' + F[2]
                if key in control_db: continue
                if key not in junc2list: junc2list[key] = 1
                

    temp_id = 0
    hout = open(output_file + ".tmp.unsorted.txt", 'w')
    for SJ_file in SJ_file_list:
        with open(SJ_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                if F[0] + '\t' + F[1] + '\t' + F[2] in junc2list:
                    print >> hout, F[0] + '\t' + F[1] + '\t' + F[2] + '\t' + str(temp_id) + '\t' + F[6]
      
            temp_id = temp_id + 1 

    hout.close()


    hout = open(output_file + '.tmp.sorted.txt', 'w')
    subprocess.call(["sort", "-k1,1", "-k2,2n", "-k3,3n", output_file + ".tmp.unsorted.txt"], stdout = hout)
    hout.close()


    temp_chr = ""
    temp_start = ""
    temp_end = ""
    temp_count = ["0"] * temp_id
    hout = open(output_file, 'w')
    with open(output_file + '.tmp.sorted.txt', 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            if F[1] != temp_start or F[2] != temp_end or F[0] != temp_chr:

                # if not the first line 
                if temp_chr != "":
                    print >> hout, temp_chr + '\t' + temp_start + '\t' + temp_end + '\t' + ','.join(temp_count)

                temp_chr = F[0]
                temp_start = F[1]
                temp_end = F[2]
                temp_count = ["0"] * temp_id


            temp_count[int(F[3])] = F[4]

    # last check 

    if temp_chr != "":
        print >> hout, temp_chr + '\t' + temp_start + '\t' + temp_end + '\t' + ','.join(temp_count)

    hout.close()
 
    # remove intermediate files
    subprocess.call(["rm", "-rf", output_file + ".tmp.unsorted.txt"])
    subprocess.call(["rm", "-rf", output_file + ".tmp.sorted.txt"])

    # if control_file is not None:
    # control_db.close()



def merge_intron_retention(IR_file_list, output_file, control_file, ratio_thres, num_thres):

    # get control intron retention info
    control_db = {}
    if control_file is not None:
        with gzip.open(control_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                control_db[F[0] + '\t' + F[1]] = 1


    # list up junctions to pick up
    intron_retention2list = {}
    header2ind = {}
    target_header = ["Chr", "Boundary_Pos", "Gene_Symbol", "Motif_Type", "Strand",
                     "Junction_List", "Gene_ID_List", "Exon_Num_List"]

    for IR_file in IR_file_list:
        with open(IR_file, 'r') as hin:
            header = hin.readline().rstrip('\n').split('\t')
            for (i, cname) in enumerate(header):
                header2ind[cname] = i

            for line in hin:
                F = line.rstrip('\n').split('\t')
                if int(F[header2ind["Intron_Retention_Read_Count"]]) < num_thres: continue
                ratio = 0
                if F[header2ind["Edge_Read_Count"]] != "0":
                    ratio = float(F[header2ind["Intron_Retention_Read_Count"]]) / float(F[header2ind["Edge_Read_Count"]])
                if ratio < ratio_thres: continue

                # check the existence in control
                if F[0] + '\t' + F[1] in control_db: continue

                key = '\t'.join([F[header2ind[x]] for x in target_header])

                if key not in intron_retention2list: intron_retention2list[key] = 1


    temp_id = 0
    hout = open(output_file + ".tmp.unsorted.txt", 'w')
    for IR_file in IR_file_list:
        with open(IR_file, 'r') as hin:
            header = hin.readline().rstrip('\n').split('\t')

            for line in hin:
                F = line.rstrip('\n').split('\t')
                key = '\t'.join([F[header2ind[x]] for x in target_header])
                if key in intron_retention2list:
                    print >> hout, key + '\t' + str(temp_id) + '\t' + F[header2ind["Intron_Retention_Read_Count"]]

        temp_id = temp_id + 1

    hout.close()


    hout = open(output_file + '.tmp.sorted.txt', 'w')
    subprocess.call(["sort", "-k1,1", "-k2,2n", output_file + ".tmp.unsorted.txt"], stdout = hout)
    hout.close()


    temp_chr = ""
    temp_pos = ""
    temp_key = ""
    temp_count = ["0"] * temp_id
    hout = open(output_file, 'w')
    print >> hout, '\t'.join(target_header) + '\t' + "Read_Count_Vector"

    with open(output_file + '.tmp.sorted.txt', 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            if F[1] != temp_pos or F[0] != temp_chr:

                # if not the first line 
                if temp_chr != "":
                    print >> hout, temp_key + '\t' + ','.join(temp_count)

                temp_chr = F[0]
                temp_pos = F[1]
                temp_key = '\t'.join([F[header2ind[x]] for x in target_header])
                temp_count = ["0"] * temp_id

            temp_count[int(F[8])] = F[9]

    
    # last check 
    if temp_key != "":
        print >> hout, temp_key + '\t' + ','.join(temp_count) 

    hout.close()
 
    # remove intermediate files
    subprocess.call(["rm", "-rf", output_file + ".tmp.unsorted.txt"])
    subprocess.call(["rm", "-rf", output_file + ".tmp.sorted.txt"])

    # if control_file is not None:
    # control_db.close()


def merge_chimera(chimera_file_list, output_file, control_file, num_thres, overhang_thres):

    # list up junctions to pick up
    chimera2list = {}
    header2ind = {}
    target_header = ["Chr_1", "Pos_1", "Dir_1", "Chr_2", "Pos_2", "Dir_2", "Inserted_Seq"]

    for chimera_file in chimera_file_list:
        with open(chimera_file, 'r') as hin:
            header = hin.readline().rstrip('\n').split('\t')
            for (i, cname) in enumerate(header):
                header2ind[cname] = i

            for line in hin:
                F = line.rstrip('\n').split('\t')
                if int(F[header2ind["Read_Pair_Num"]]) < num_thres: continue
                if int(F[header2ind["Max_Over_Hang_1"]]) < overhang_thres: continue
                if int(F[header2ind["Max_Over_Hang_2"]]) < overhang_thres: continue

                key = '\t'.join([F[header2ind[x]] for x in target_header])
                if key not in chimera2list: chimera2list[key] = 1


    temp_id = 0
    hout = open(output_file + ".tmp.unsorted.txt", 'w')
    for chimera_file in chimera_file_list:
        with open(chimera_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                key = '\t'.join([F[header2ind[x]] for x in target_header])
                if key in chimera2list:
                    print >> hout, key + '\t' + str(temp_id) + '\t' + F[header2ind["Read_Pair_Num"]]

        temp_id = temp_id + 1

    hout.close()


    hout = open(output_file + '.tmp.sorted.txt', 'w')
    subprocess.call(["sort", "-k1,1", "-k2,2n", "-k4,4", "-k5,5n", output_file + ".tmp.unsorted.txt"], stdout = hout)
    hout.close()

    if control_file is not None:
        control_db = pysam.TabixFile(control_file)
 
    temp_chr = ""
    temp_pos = ""
    temp_key = ""
    temp_count = ["0"] * temp_id
    hout = open(output_file, 'w')
    print >> hout, '\t'.join(target_header) + '\t' + "Read_Count_Vector"

    with open(output_file + '.tmp.sorted.txt', 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            key = '\t'.join([F[header2ind[x]] for x in target_header])
            if key != temp_key: 
 
                # if not the first line 
                if temp_key != "":
 
                    # skip if the junction is included in the control file
                    control_flag = 0
                    if control_file is not None:
                        tabixErrorFlag = 0
                        try:
                            records = control_db.fetch(temp_chr, int(temp_pos) - 5, int(temp_pos) + 5)
                        except Exception as inst:
                            # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
                            # tabixErrorMsg = str(inst.args)
                            tabixErrorFlag = 1
 
                        if tabixErrorFlag == 0:
                            for record_line in records:
                                record = record_line.split('\t')
                                record_key = '\t'.join([record[header2ind[x]] for x in target_header])                            
                                if temp_key == record_key:
                                    control_flag = 1
 
                    if control_flag == 0:
                        print >> hout, temp_key + '\t' + ','.join(temp_count)

                temp_chr = F[0]
                temp_pos = F[1] 
                temp_key = key
                temp_count = ["0"] * temp_id
 
            temp_count[int(F[7])] = F[8]

    # last check 
    # skip if the junction is included in the control file
    control_flag = 0
    if control_file is not None:
        tabixErrorFlag = 0
        try:
            records = control_db.fetch(temp_chr, int(temp_pos) - 5, int(temp_pos) + 5)
        except Exception as inst:
            # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
            # tabixErrorMsg = str(inst.args)
            tabixErrorFlag = 1
 
        if tabixErrorFlag == 0:
            for record_line in records:
                record = record_line.split('\t')
                record_key = '\t'.join([record[header2ind[x]] for x in target_header])
                if temp_key == record_key:
                    control_flag = 1
 
    if control_flag == 0:
        print >> hout, temp_key + '\t' + ','.join(temp_count)

    hout.close()
 
    # remove intermediate files
    subprocess.call(["rm", "-rf", output_file + ".tmp.unsorted.txt"])
    subprocess.call(["rm", "-rf", output_file + ".tmp.sorted.txt"])
 
    if control_file is not None:
        control_db.close()


def merge_mut(mutation_file_list, output_file):


    mut2sample = {}
    sample_ind = 0
    for mut_file in mutation_file_list:
        sample_ind = sample_ind + 1
        with open(mut_file, 'r') as hin2:
            for line2 in hin2:
                F2 = line2.rstrip('\n').split('\t')
                if F2[0].startswith('#'): continue
                if F2[0] == "Chr": continue

                key = '\t'.join(F2[0:5])
       
                if key not in mut2sample: 
                    mut2sample[key] = []
              
                mut2sample[key].append(str(sample_ind))

    sample_num = sample_ind

    hout = open(output_file, 'w')
    for mut in sorted(mut2sample):
        if len(mut2sample[mut]) == sample_num: continue
        print >> hout, mut + '\t' + ','.join(mut2sample[mut]) 

    hout.close()


def merge_mut2(mutation_file_list, output_file, reference):


    mut2sample = {}
    sample_ind = 0
    for mut_file in mutation_file_list:
        sample_ind = sample_ind + 1
        is_vcf = True if mut_file.endswith(".vcf") or mut_file.endswith(".vcf.gz") else False
        hin2 = gzip.open(mut_file, 'r') if mut_file.endswith(".gz") else open(mut_file, 'r')

        for line2 in hin2:
            F2 = line2.rstrip('\n').split('\t')
            if F2[0].startswith('#'): continue
            if F2[0] == "Chr": continue

            if is_vcf == False:
                pos, ref, alt = F2[1], F2[3], F2[4]
        
                # insertion
                if F2[3] == "-":
                    # get the sequence for the reference base
                    seq = ""    
                    for item in pysam.faidx(reference, F2[0] + ":" + str(F2[1]) + "-" + str(F2[1])):
                        seq = seq + item.rstrip('\n')
                    seq = seq.replace('>', '')
                    seq = seq.replace(F2[0] + ":" + str(F2[1]) + "-" + str(F2[1]), '')
                    ref, alt = seq, seq + F2[4]

                # deletion
                if F2[4] == "-":
                    # get the sequence for the reference base
                    seq = ""    
                    for item in pysam.faidx(reference, F2[0] + ":" + str(int(F2[1]) - 1) + "-" + str(int(F2[1]) - 1)):
                        seq = seq + item.rstrip('\n')
                    seq = seq.replace('>', '')
                    seq = seq.replace(F2[0] + ":" + str(int(F2[1]) - 1) + "-" + str(int(F2[1]) - 1), '')
                    pos, ref, alt = str(int(F2[1]) - 1), seq + F2[3], seq

                QUAL = 60
                INFO = "SOMATIC"

                key = '\t'.join([F2[0], pos, '.', ref, alt, str(QUAL), "PASS", INFO])

            else:

                key = '\t'.join(F2[0:8])

            if key not in mut2sample:
                mut2sample[key] = []

            mut2sample[key].append(str(sample_ind))

    sample_num = sample_ind

    hout = open(output_file, 'w')
    for mut in sorted(mut2sample):
        if len(mut2sample[mut]) == sample_num: continue
        print >> hout, mut + '\t' + ','.join(mut2sample[mut])

    hout.close()


def merge_sv(sv_file_list, output_file):

    sv2sample = {}
    sample_num = "1"
    for sv_file in sv_file_list:
        with open(sv_file, 'r') as hin2:
            for line2 in hin2:
                F2 = line2.rstrip('\n').split('\t')
                if F2[0].startswith('#'): continue
                if F2[0] == "Chr_1": continue
                if F2[0] == "": continue

                key = '\t'.join(F2[0:7])

                if key not in sv2sample:
                    sv2sample[key] = []

                sv2sample[key].append(sample_num)

        sample_num = str(int(sample_num) + 1)

    hout = open(output_file, 'w')
    for sv in sorted(sv2sample):
        print >> hout, sv + '\t' + ','.join(sv2sample[sv])

    hout.close()

def merge_SJ_IR_files(SJ_input_file, IR_input_file, output_file):

    header2ind = {}
    header = ""
    hout = open(output_file + ".unsorted", 'w')

    with open(SJ_input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        for line in hin:
            F = line.rstrip('\n').split('\t')
            genes = F[header2ind["Gene_1"]].split(';') + F[header2ind["Gene_2"]].split(';')
            genes = sorted(list(set(genes)))

            if "---" in genes: genes.remove("---")
            if len(genes) > 0:
                genes_nm = filter(lambda x: x.find("(NM_") > 0, genes)
                if len(genes_nm) > 0: genes = genes_nm

            if len(genes) > 0:
                genes_single = filter(lambda x: x.find("-") == -1, genes)
                if len(genes_single) > 0: genes = genes_single
 
            gene = genes[0]
            gene = re.sub(r"\(N[MR]_\d+\)", "", gene)

            splicing_key = F[header2ind["SJ_1"]] + ':' + F[header2ind["SJ_2"]] + '-' + F[header2ind["SJ_3"]]

            print >> hout, gene + '\t' + splicing_key + '\t' + F[header2ind["Splicing_Class"]] + '\t' + F[header2ind["Is_Inframe"]] + '\t' + \
                            F[header2ind["SJ_4"]] + '\t' + F[header2ind["Mutation_Key"]] + '\t' + F[header2ind["Motif_Pos"]] + '\t' + \
                            F[header2ind["Mutation_Type"]] + '\t' + F[header2ind["Is_Canonical"]]


    with open(IR_input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        for line in hin:
            F = line.rstrip('\n').split('\t')
            splicing_key = F[header2ind["Chr"]] + ':' + F[header2ind["Boundary_Pos"]] + '-' + F[header2ind["Boundary_Pos"]]
            splicing_class = "Intron retention" if F[header2ind["Intron_Retention_Type"]] == "Direct impact" else "Opposite side intron retention"
            print >> hout, F[header2ind["Gene_Symbol"]] + '\t' + splicing_key + '\t' + splicing_class + '\t' + '---' + '\t' + \
                           F[header2ind["Read_Count_Vector"]] + '\t' + F[header2ind["Mutation_Key"]] + '\t' + \
                           F[header2ind["Motif_Pos"]] + '\t' + F[header2ind["Mutation_Type"]] + '\t' + F[header2ind["Is_Canonical"]]

    hout.close()

    hout = open(output_file, 'w')
    print >> hout, '\t'.join(["Gene_Symbol", "Splicing_Key", "Splicing_Class", "Is_Inframe", "Read_Counts",
                              "Mutation_Key", "Motif_Pos", "Mutation_Type", "Is_Canonical"])
    hout.close()

    hout = open(output_file, 'a')
    subprocess.call(["sort", "-k1", output_file + ".unsorted"], stdout = hout)
    hout.close()

    subprocess.call(["rm", "-rf", output_file + ".unsorted"])


def get_sv_type(sv_key):

    sv_info = sv_key.split(',')
    sv_type = "dummy"
    if sv_info[0] != sv_info[3]:
        sv_type = "translocation"
    elif sv_info[2] == '+' and sv_info[5] == '-':
        sv_type = "deletion"
    elif sv_info[2] == '-' and sv_info[5] == '+':
        sv_type = "tandem_duplication"
    else:
        sv_type = "inversion"
    return(sv_type)


def merge_SJ_IR_chimera_files_sv(SJ_input_file, IR_input_file, chimera_input_file, output_file):

    header2ind = {}
    header = ""
    hout = open(output_file + ".unsorted", 'w')

    with open(SJ_input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        for line in hin:
            F = line.rstrip('\n').split('\t')
            gene1 = gene_filter(F[header2ind["Gene_1"]].split(';'))
            gene2 = gene_filter(F[header2ind["Gene_2"]].split(';'))
            gene = list(set(gene1) & set(gene2))
            if len(gene) == 0: continue

            splicing_key = F[header2ind["SJ_1"]] + ':' + F[header2ind["SJ_2"]] + '-' + F[header2ind["SJ_3"]]
            print >> hout, gene[0] + '\t' + splicing_key + '\t' + F[header2ind["Splicing_Class"]] + '\t' + F[header2ind["Is_Inframe"]] + '\t' + \
                            F[header2ind["SJ_4"]] + '\t' + F[header2ind["SV_Key"]] + '\t' + get_sv_type(F[header2ind["SV_Key"]])


    with open(IR_input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        for line in hin:
            F = line.rstrip('\n').split('\t')
            splicing_key = F[header2ind["Chr"]] + ':' + F[header2ind["Boundary_Pos"]] + '-' + F[header2ind["Boundary_Pos"]]
            splicing_class = "Intron retention"

            print >> hout, F[header2ind["Gene_Symbol"]] + '\t' + splicing_key + '\t' + splicing_class + '\t' + '---' + '\t' + \
                           F[header2ind["Read_Count_Vector"]] + '\t' + F[header2ind["SV_Key"]] + '\t' + get_sv_type(F[header2ind["SV_Key"]]) 

    with open(chimera_input_file, 'r') as hin: 

        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i


        for line in hin: 
            F = line.rstrip('\n').split('\t')

            gene1 = gene_filter(F[header2ind["Gene_1"]].split(';'))
            gene2 = gene_filter(F[header2ind["Gene_2"]].split(';'))
            gene = list(set(gene1) & set(gene2))
            if len(gene) == 0: continue

            # currently, only consider exon_reusage and unspliced_chimera
            if F[header2ind["Chimera_Class"]] not in ["Exon reusage", "Unspliced chimera"]: continue

            splicing_key = ','.join([F[header2ind[x]] for x in ["Chr_1", "Pos_1", "Dir_1", "Chr_2", "Pos_2", "Dir_2", "Inserted_Seq"]])
            print >> hout, gene[0] + '\t' + splicing_key + '\t' + F[header2ind["Chimera_Class"]] + '\t' + "---" + '\t' + \
                           F[header2ind["Read_Count_Vector"]] + '\t' + F[header2ind["SV_Key"]] + '\t' + get_sv_type(F[header2ind["SV_Key"]]) 

    hout.close()

    hout = open(output_file, 'w')
    print >> hout, '\t'.join(["Gene_Symbol", "Splicing_Key", "Splicing_Class", "Is_Inframe", "Read_Counts", "SV_Key", "SV_Type"])
    hout.close()

    hout = open(output_file, 'a')
    subprocess.call(["sort", "-k1", output_file + ".unsorted"], stdout = hout)
    hout.close()

    subprocess.call(["rm", "-rf", output_file + ".unsorted"])


def add_gene_symbol(input_file, output_file):

    header2ind = {}
    header = ""
    hout0 = open(output_file + ".tmp0", 'w')
    hout1 = open(output_file + ".tmp1", 'w')
    with open(input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i

        print >> hout0, "Gene_Symbol" + '\t' + '\t'.join(header)

        for line in hin:
            F = line.rstrip('\n').split('\t')
            genes = F[header2ind["Gene_1"]].split(';') + F[header2ind["Gene_2"]].split(';')
            genes = list(set(genes))

            if "---" in genes: genes.remove("---")
            if len(genes) > 0: 
                genes_nm = filter(lambda x: x.find("(NM_") > 0, genes)
                if len(genes_nm) > 0: genes = genes_nm

            gene = genes[0]
            gene = re.sub(r"\(N[MR]_\d+\)", "", gene)

            print >> hout1, gene + '\t' + '\t'.join(F)

    hout0.close()
    hout1.close()

    hout2 = open(output_file + ".tmp2", 'w')
    subprocess.call(["sort", "-k1", output_file + ".tmp1"], stdout = hout2)
    hout2.close()

    hout3 = open(output_file, 'w')
    subprocess.call(["cat", output_file + ".tmp0", output_file + ".tmp2"], stdout = hout3)
    hout3.close()

    subprocess.call(["rm", "-rf", output_file + ".tmp0"])
    subprocess.call(["rm", "-rf", output_file + ".tmp1"])
    subprocess.call(["rm", "-rf", output_file + ".tmp2"])



def gene_filter(genes):

    genes = list(set(genes))
    if "---" in genes: genes.remove("---")
    if len(genes) > 0: 
        genes_nm = filter(lambda x: x.find("(NM_") > 0, genes)
        if len(genes_nm) > 0: genes = genes_nm
   
    if len(genes) > 0: 
        genes = map(lambda x: re.sub(r"\(N[MR]_\d+\)", "", x), genes)
    else:
        genes = [] 

    return(genes)





#! /usr/bin/env python

import sys, os

class Sample_conf(object):

    def __init__(self):

        self.sample_names = []
        self.mut_files = []
        self.sv_files = [] 
        self.SJ_files = [] 
        self.IR_files = [] 
        self.chimera_files = []
        self.weights = [] 

    def parse_file(self, file_path, sv_mode):

        header2ind = {}
        with open(file_path) as hin:

            # read header
            header = hin.readline().rstrip('\n').split('\t')
            for (i, cname) in enumerate(header):
    
                if sv_mode == False:
                    if cname not in ["Sample_Name", "Mutation_File", "SJ_File", "IR_File", "Weight"]:
                        print >> sys.stderr, "header should any of Sample_Name, Mutation_File, SJ_File, IR_File or Weight"
                        sys.exit(1)
                else:
                    if cname not in ["Sample_Name", "SV_File", "SJ_File", "IR_File", "Chimera_File", "Weight"]:
                        print >> sys.stderr, "header should any of Sample_Name, SV_File, SJ_File, IR_File, Chimera_File or Weight"
                        sys.exit(1)
                header2ind[cname] = i
        
            # check header
            if "Sample_Name" not in header2ind:
                print >> sys.stderr, "Sample_Name column should be included in the sample config file"
                sys.exit(1)

            if sv_mode == False and "Mutation_File" not in header2ind:
                print >> sys.stderr, "Mutation_File column should be included in the sample config file"
                sys.exit(1)

            if sv_mode == True and "SV_File" not in header2ind:
                print >> sys.stderr, "SV_File column should be included in the sample config file"
                sys.exit(1)

            if sv_mode == False:
                if "SJ_File" not in header2ind and "IR_File" not in header2ind:
                    print >> sys.stderr, "At leaset one of SJ_File column or IR_File column should be included inthe smaple config file"
                    sys.exit(1)
                if "SJ_File" not in header2ind and "IR_File" not in header2ind and "Chimera_File" not in header2ind:
                    print >> sys.stderr, "At leaset one of SJ_File, IR_File or Chimera_File column should be included inthe smaple config file"
                    sys.exit(1)

        
            for line in hin:
                F = line.rstrip('\n').split('\t')

                self.sample_names.append(F[header2ind["Sample_Name"]])

                if sv_mode == False:
                    if not os.path.exists(F[header2ind["Mutation_File"]]):
                        print >> sys.stderr, F[header2ind["Mutation_File"]] + " does not exist"
                        sys.exit(1)
                    else:
                        self.mut_files.append(F[header2ind["Mutation_File"]])
                else:
                    if not os.path.exists(F[header2ind["SV_File"]]):
                        print >> sys.stderr, F[header2ind["SV_File"]] + " does not exist"
                        sys.exit(1)
                    else:
                        self.sv_files.append(F[header2ind["SV_File"]])

                if "SJ_File" in header2ind:
                    if not os.path.exists(F[header2ind["SJ_File"]]):
                        print >> sys.stderr, F[header2ind["SJ_File"]] + " does not exist" 
                        sys.exit(1)
                    else:
                        self.SJ_files.append(F[header2ind["SJ_File"]])

                if "IR_File" in header2ind:
                    if not os.path.exists(F[header2ind["IR_File"]]):
                        print >> sys.stderr, F[header2ind["IR_File"]] + " does not exist"
                        sys.exit(1)
                    else:
                        self.IR_files.append(F[header2ind["IR_File"]])

                if "Chimera_File" in header2ind:
                    if not os.path.exists(F[header2ind["Chimera_File"]]):
                        print >> sys.stderr, F[header2ind["Chimera_File"]] + " does not exist"
                        sys.exit(1)
                    else:
                        self.chimera_files.append(F[header2ind["Chimera_File"]])

                if "Weight" in header2ind:
                    self.weights.append(float(F[header2ind["Weight"]]))
                    # self.weights.append(0.10)
                else:
                    self.weights.append(1.0)



#! /usr/bin/env python

import copy, math, random
import utils
from sav import Sav

class Network(object):

    def __init__(self, gene, mutation_status, splicing_counts, link2info, sample_list, weight_vector):
        self.gene = gene
        self.mutation_status = mutation_status
        self.splicing_counts = splicing_counts
        self.link_vector = link2info.keys()
        self.link2info = link2info
        self.sample_list = sample_list
        self.sample_num = len(sample_list)
        self.weight_vector = weight_vector

        self.pruned_link_vector = []

        self.link_vector2effect_size = {}
        # self.link_vector2median_count = {}

        self.link_vector2active_quartile_count = {}
        self.link_vector2inactive_quartile_count = {}

        self.clustered_link_vector = []

        self.mut2log_BF = {}
        self.mut2significant_links = {}


    def prune_link_vector(self, margin, active_zero_filter_prob, inactive_nonzero_filter_prob):

        self.__link_effect_size_scan()
        # self.__link_median_count_scan()
        self.__link_active_quartile_count_scan(active_zero_filter_prob)
        self.__link_inactive_quartile_count_scan(inactive_nonzero_filter_prob)
        # self.pruned_link_vector = [x for x in self.link_vector if self.link_vector2effect_size[x] >= margin and self.link_vector2median_count[x] == 0]
        self.pruned_link_vector = [x for x in self.link_vector if self.link_vector2effect_size[x] >= margin and \
                                                                  self.link_vector2active_quartile_count[x] > 0 and \
                                                                  self.link_vector2inactive_quartile_count[x] == 0]


    def cluster_link_vector(self):

        for sub_cluster_link in self.__simple_cluster_link():

            if len(sub_cluster_link) > 15:

                sub_cluster_effect_size_vector = [self.link_vector2effect_size[x] for x in sub_cluster_link]
                temp_margin = sorted(sub_cluster_effect_size_vector, reverse=True)[15]
                sub_cluster_link = [sub_cluster_link[i] for i in range(len(sub_cluster_link)) if sub_cluster_effect_size_vector[i] > temp_margin]

            # may be removed at the above filtering step
            if len(sub_cluster_link) > 0:
                self.clustered_link_vector.append(sub_cluster_link)

    
    def get_averaged_bayes_factors(self, alpha0, beta0, alpha1, beta1):

        for sub_cluster_link in self.clustered_link_vector:

            conf_dim = len(sub_cluster_link)

            mut2log_ML_null = {}
            mut2log_ML_nonnull = {}
            mut2log_ML_nonnull_max = {}
            mut2conf_max = {}
            mut2log_BF = {}

            # get mutations associated with the current splicing
            for mut_id, _ in sub_cluster_link:
                mut2log_ML_null[mut_id] = [] 
                mut2log_ML_nonnull[mut_id] = []
                mut2log_ML_nonnull_max[mut_id] = float("-inf") 
                mut2conf_max[mut_id] = [0] * conf_dim
                mut2log_BF[mut_id] = float("-inf")


            for conf in sorted(utils.generate_configurations(conf_dim)):

                # get active mutation in the configuration in consideration
                active_mut_list = []
                # get mutations associated with the current splicing
                for j in range(len(sub_cluster_link)):
                    if int(conf[j]) == 0: continue
                    mut_id, sp_id = sub_cluster_link[j]
                    active_mut_list.append(mut_id)

                log_ML = self.__get_log_marginal_likelihood(sub_cluster_link, conf, alpha0, beta0, alpha1, beta1)

                for mut_id in mut2log_ML_null:
                    if mut_id in active_mut_list:
                        mut2log_ML_nonnull[mut_id].append(log_ML)
                        if log_ML > mut2log_ML_nonnull_max[mut_id]:
                            mut2log_ML_nonnull_max[mut_id] = log_ML
                            mut2conf_max[mut_id] = conf
                    else:
                        mut2log_ML_null[mut_id].append(log_ML)


            for mut_id in mut2log_ML_null:

                self.mut2log_BF[mut_id] = utils.soft_max(mut2log_ML_nonnull[mut_id]) - utils.soft_max(mut2log_ML_null[mut_id]) - \
                                            math.log(len(mut2log_ML_nonnull[mut_id])) + math.log(len(mut2log_ML_null[mut_id]))

                max_conf = mut2conf_max[mut_id]
                significant_links = [sub_cluster_link[i] for i in range(len(max_conf)) if max_conf[i] == 1]
                self.mut2significant_links[mut_id] = significant_links


    def export_to_savs(self, log_BF_thres, sample_num_thres = 1):
   
        sav_list = []
        all_active_sample_list = []

        for mut_id in self.mut2log_BF:
            if self.mut2log_BF[mut_id] < log_BF_thres: continue

            active_sample_list = [self.sample_list[i] for i in self.mutation_status[mut_id]]
            all_active_sample_list = all_active_sample_list + active_sample_list

            active_splicing_counts_vector = []
            active_link_info_vector = [] 
            for active_link in self.mut2significant_links[mut_id]:
                cur_mut_id, cur_sp_id = active_link
                if cur_mut_id != mut_id: continue

                cur_splicing_counts = self.splicing_counts[cur_sp_id]
                active_splicing_counts_vector.append([cur_splicing_counts[i] for i in self.mutation_status[mut_id]])
                active_link_info_vector.append(self.link2info[active_link])

            tsav = Sav(self.gene, active_sample_list, active_link_info_vector, active_splicing_counts_vector, self.mut2log_BF[mut_id])
            sav_list.append(tsav)

        all_active_sample_list = list(set(all_active_sample_list))
        if len(all_active_sample_list) >= sample_num_thres:
            return sav_list
        else:
            return []
               
        
    def execute_permutation(self, seed):

        for mutation in self.mutation_status:
            shuffled_samples = [seed[x] for x in self.mutation_status[mutation]]
            self.mutation_status[mutation] = shuffled_samples

        
    def __link_effect_size_scan(self, pseudo_count = 0.1):

        # simple check for each link
        for mut_id, sp_id in self.link_vector:
            cur_splicing_count_vector = self.splicing_counts[sp_id] 

            # extract samples with the mutation of the link in consideration
            mut_vector = [0] * self.sample_num
            for sample_id in self.mutation_status[mut_id]:
                mut_vector[sample_id] = 1

            weight_sum_null = sum([float(self.weight_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 0])
            weight_sum_target = sum([float(self.weight_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 1])

            count_sum_null = sum([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 0])
            count_sum_target = sum([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 1])

            mean_null = float(count_sum_null) / weight_sum_null if weight_sum_null > 0.0 and count_sum_null > 0 else 0.0
            mean_target = float(count_sum_target) / weight_sum_target if weight_sum_target > 0 and count_sum_target > 0 else 0.0
    
            self.link_vector2effect_size[(mut_id, sp_id)] = (float(mean_target) + pseudo_count) / (float(mean_null) + pseudo_count)



    def __link_active_quartile_count_scan(self, active_zero_filter_prob = 0.5):

        # simple check for each link
        for mut_id, sp_id in self.link_vector:
            # cur_mut_id, cur_sp_id = self.link_vector[i]
            cur_splicing_count_vector = self.splicing_counts[sp_id]

            # extract samples with the mutation of the link in consideration
            mut_vector = [0] * self.sample_num
            for sample_id in self.mutation_status[mut_id]:
                mut_vector[sample_id] = 1

            # self.link_vector2median_count[(mut_id, sp_id)] = utils.median([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 0])
            self.link_vector2active_quartile_count[(mut_id, sp_id)] = utils.quartile([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 1], active_zero_filter_prob)


    """
    # def __link_median_count_scan(self):
    def __link_inactive_quartile_count_scan(self, inactive_nonzero_filter_prob = 0.5):

        # simple check for each link
        for mut_id, sp_id in self.link_vector:
            # cur_mut_id, cur_sp_id = self.link_vector[i]
            cur_splicing_count_vector = self.splicing_counts[sp_id]

            # extract samples with the mutation of the link in consideration
            mut_vector = [0] * self.sample_num
            for sample_id in self.mutation_status[mut_id]:
                mut_vector[sample_id] = 1

            # self.link_vector2median_count[(mut_id, sp_id)] = utils.median([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 0])
            self.link_vector2inactive_quartile_count[(mut_id, sp_id)] = utils.quartile([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if mut_vector[j] == 0], inactive_nonzero_filter_prob)
    """

    def __link_inactive_quartile_count_scan(self, inactive_nonzero_filter_prob = 0.5):

        sp_id2mut_vector = {}
        for mut_id, sp_id, in self.link_vector:

            if sp_id not in sp_id2mut_vector: sp_id2mut_vector[sp_id] = [0] * self.sample_num
            for sample_id in self.mutation_status[mut_id]:
                sp_id2mut_vector[sp_id][sample_id] = 1

            
        for mut_id, sp_id, in self.link_vector:
            cur_splicing_count_vector = self.splicing_counts[sp_id]
            self.link_vector2inactive_quartile_count[(mut_id, sp_id)] = utils.quartile([int(cur_splicing_count_vector[j]) for j in range(self.sample_num) if sp_id2mut_vector[sp_id][j] == 0], inactive_nonzero_filter_prob)



    # simply cluster links by their topologies
    def __simple_cluster_link(self):

        simple_clustered_link = []
        mut_id2sp_id = {}
        for mut_id, sp_id in self.pruned_link_vector:
            if mut_id not in mut_id2sp_id: mut_id2sp_id[mut_id] = []
            mut_id2sp_id[mut_id].append(sp_id)

        clustered_link = []
        mut_ids = mut_id2sp_id.keys()

        # first, append all the mut_ids to the remaining_ids
        remaining_ids = copy.deepcopy(mut_ids) # deep copy

        for i in range(len(mut_ids)):

            # start from the i-th mut_id if this is still remaining
            if mut_ids[i] not in remaining_ids: continue
            clustered_ids = [mut_ids[i]]

            no_more_cluster = 0
            while no_more_cluster == 0:

                no_more_cluster = 1
                for check_mut_id in remaining_ids:
                    if check_mut_id in clustered_ids: continue
                    for cur_clustered_mut_id in clustered_ids:
                        # check whether there is overlapping splicing aberrations
                        if len(set(mut_id2sp_id[cur_clustered_mut_id]) & set(mut_id2sp_id[check_mut_id])) > 0:
                            clustered_ids.append(check_mut_id)
                            no_more_cluster = 0
                            break


            tmp_clink_vector = []
            for c_mut_id in sorted(clustered_ids):
                tmp_clink_vector = tmp_clink_vector + [(c_mut_id, x) for x in mut_id2sp_id[c_mut_id]]
                remaining_ids.remove(c_mut_id)

            # simple_clustered_link.append(tmp_clink_vector)
            yield tmp_clink_vector

        # return simple_clustered_link


    def __get_log_marginal_likelihood(self, sub_cluster_link, configuration_vector, alpha0, beta0, alpha1, beta1):

        active_sp_list = []
        # get mutations associated with the current splicing
        for mut_id, sp_id in sub_cluster_link:
            active_sp_list.append(sp_id)


        log_marginal_likelihood = 0
        for sp_id in range(len(self.splicing_counts)):

            if sp_id not in active_sp_list: continue

            cur_splicing_count_vector = self.splicing_counts[sp_id]
            active_mut_list = []

            # get mutations associated with the current splicing
            for link_id in range(len(sub_cluster_link)):
                if configuration_vector[link_id] == 0: continue
                mut_id, link_sp_id = sub_cluster_link[link_id]
                if link_sp_id == sp_id: active_mut_list.append(mut_id)

            # param_num = param_num + len(active_mut_list) + 1
            active_mut_vector = [0] * self.sample_num

            # set mutation status
            for mut_id in self.mutation_status: 
                if mut_id in active_mut_list:
                    sample_id_vector = self.mutation_status[mut_id]
                    for sample_id in sample_id_vector:
                        active_mut_vector[sample_id] = 1

            # for inactive mutations
            sample_sum = len([n for n in range(self.sample_num) if active_mut_vector[n] == 0])
            count_sum = sum([int(cur_splicing_count_vector[n]) for n in range(self.sample_num) if active_mut_vector[n] == 0])
            weight_sum = sum([self.weight_vector[n] for n in range(self.sample_num) if active_mut_vector[n] == 0])

            partial_log_marginal_likelihood = 0
            if sample_sum > 0:
                partial_log_marginal_likelihood = partial_log_marginal_likelihood + math.lgamma(count_sum + alpha0) - math.lgamma(alpha0) + \
                                                    alpha0 * math.log(beta0) - (count_sum + alpha0) * math.log(weight_sum + beta0)

            for n in range(self.sample_num):
                if active_mut_vector[n] == 0: continue
                partial_log_marginal_likelihood = partial_log_marginal_likelihood + math.lgamma(int(cur_splicing_count_vector[n]) + alpha1) - math.lgamma(alpha1) + \
                                                    alpha1 * math.log(beta1) - (int(cur_splicing_count_vector[n]) + alpha1) * math.log(self.weight_vector[n] + beta1)

            log_marginal_likelihood = log_marginal_likelihood + partial_log_marginal_likelihood

        return(log_marginal_likelihood)


if __name__ == "__main__":

    from collections import namedtuple

    mutation_status = {0: [2]}
    splicing_counts = [[0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                       [0,0,19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                       [0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]]
    # link_vector = [(0, 0), (0, 1), (0, 2)]
    # sample_num = 26
    sample_list = ["A427", "A549", "ABC-1", "H1299", "H1437", "H1648", "H1650", "H1703", "H1819", 
                   "H1975", "H2126", "H2228", "H2347", "H322", "II-18", "LC2_ad", "PC-14", "PC-3", 
                   "PC-7", "PC-9", "RERF-LC-Ad1", "RERF-LC-Ad2", "RERF-LC-KJ", "RERF-LC-MS", "RERF-LC-OK", "VMRC-LCD"]

    weight_vector = [5.2935, 2.5843, 4.6718, 6.7032, 6.3449, 4.8819, 3.2902, 11.6925, 9.7603, 4.4712, 5.8023, 7.3304, 6.6022,
                     6.8118, 8.0286, 5.5164, 6.7205, 6.3547, 6.5036, 4.2754, 6.989, 5.63, 7.514, 6.6156, 4.1002, 6.0268]

    Link_info_mut = namedtuple("Link_info_mut", ("Mutation_Key", "Motif_Pos", "Mutation_Type", "Is_Canonical", "Splicing_Key", "Splicing_Class", "Is_Inframe"))
    link2info = {(0, 0): Link_info_mut("12,475272,T,A", "12:475270-475276,-", "splicing acceptor disruption", "canonical", "12:461491-493196", "exon-skip", "in-frame"),
                 (0, 1): Link_info_mut("12,475272,T,A", "12:475270-475276,-", "splicing acceptor disruption", "canonical", "12:472264-493196", "exon-skip", "in-frame"),
                 (0, 2): Link_info_mut("12,475272,T,A", "12:475270-475276,-", "splicing acceptor disruption", "canonical", "12:475260-493196", "alternative-3'-splice-site", "in-frame")}


    network = Network("KDM5A", mutation_status, splicing_counts, link2info, sample_list, weight_vector)

    network.prune_link_vector(3.0, 0.5, 0.5)


    print network.link_vector2effect_size
    # print network.link_vector2median_count
    print network.link_vector2inactive_quartile_count
    print network.pruned_link_vector


    network.cluster_link_vector()
    print network.clustered_link_vector


    network.get_averaged_bayes_factors(1.0, 1.0, 1.0, 0.01)
    print network.mut2log_BF
    print network.mut2significant_links

    for sav in network.export_to_savs(3.0):
        print sav.print_records(with_fdr = False)

 



#! /usr/bin/env python

class Sav(object):

    print_header_mut = '\t'.join(["Gene_Symbol", "Sample_Name", "Mutation_Key", "Motif_Pos", "Mutation_Type", "Is_Canonical",
                                  "Splicing_Key", "Splicing_Class", "Is_Inframe", "Supporting_Read_Num", "Score", "Q_Value"])

    print_header_sv = '\t'.join(["Gene_Symbol", "Sample_Name", "SV_Key", "SV_Type",
                                  "Splicing_Key", "Splicing_Class", "Is_Inframe", "Supporting_Read_Num", "Score", "Q_Value"])
 
    def __init__(self, gene, sample_list, link_info_vector, supporting_read_num_vector, score):
        self.gene = gene
        self.sample_list = sample_list
        self.link_info_vector = link_info_vector
        self.link_num = len(link_info_vector)
        self.supporting_read_num_vector = supporting_read_num_vector
        self.score = score
        self.fdr = None

    def set_fdr(self, fdr):
        self.fdr = fdr

    def print_records(self, sv_mode = False, with_fdr = True):

        records = []
        for i in range(self.link_num):

            link_info = self.link_info_vector[i]
            supporting_read_num = self.supporting_read_num_vector[i]

            if sv_mode == False:
                print_link_info = '\t'.join([link_info.Mutation_Key, link_info.Motif_Pos, link_info.Mutation_Type,
                                             link_info.Is_Canonical, link_info.Splicing_Key, link_info.Splicing_Class, link_info.Is_Inframe])
            else:
                print_link_info = '\t'.join([link_info.SV_Key, link_info.SV_Type, link_info.Splicing_Key, link_info.Splicing_Class, link_info.Is_Inframe])

            temp_record = self.gene + '\t' + ';'.join(self.sample_list) + '\t' + print_link_info + '\t' + \
                            ';'.join([str(x) for x in supporting_read_num]) + '\t' + str(round(self.score, 4))

            if with_fdr:
                temp_record = temp_record + '\t' + str(round(self.fdr, 4))
            else:
                temp_record = temp_record + '\t' + "---"

            records.append(temp_record)

        return records




#! /usr/bin/env python

from run import *
import argparse
from version import __version__

def create_parser():

    parser = argparse.ArgumentParser(prog = "savnet")

    parser.add_argument("--version", action = "version", version = "%(prog)s " + __version__)

    parser.add_argument("sample_list_file", metavar = "sample_list.txt", default = None, type = str,
                        help = "Tab-delimited file of the cohort with the followin columns (the order can be arbitrary): \
                                Sample_Name: Sample labels, \
                                Mutation_File: Path to mutation data (vcf or ANNOVAR format), \
                                SJ_File: Path to splicing junction data (generated by STAR), \
                                IR_File: Path to intron retention count file (generated by intron_retention_utils simple_count), \
                                Weight: The value used for negating the diversity of the numbers of total RNA-seq reads among samples (optional).")

    parser.add_argument("output_prefix", metavar = "output_prefix", default = None, type = str, 
                        help = "the prefix of the output")

    parser.add_argument("--grc", default = False, action = 'store_true',
                        help = "Deprecated. This is not used any more. Convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    parser.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                        help = "the genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    parser.add_argument("--reference", metavar = "reference.fa", default = None, type = str,
                        help = "the path to the reference genome sequence")

    parser.add_argument("--sv", action='store_true',
                        help = "analysis structural variation file")

#     parser.add_argument("--branchpoint", action='store_true',
#                         help = "include branchpoint to the analysis")

    parser.add_argument("--donor_size", metavar = "donor_size", default = "3,6", type = str,
                        help = "splicing donor site size (exonic region size, intronic region size) (default: %(default)s)")

    parser.add_argument("--acceptor_size", metavar = "acceptor_size", default = "6,1", type = str,
                        help = "splicing donor site size (intronic region size, exonic region size) (default: %(default)s)")

    parser.add_argument("--SJ_pooled_control_file", default = None, type = str,
                        help = "the path to control data created by junc_utils merge_control (default: %(default)s)")

    parser.add_argument("--IR_pooled_control_file", default = None, type = str,
                        help = "the path to control data created by intron_retention_utils merge_control (default: %(default)s)")

    parser.add_argument("--chimera_pooled_control_file", default = None, type = str,
                        help = "the path to control data created by chimera_utils merge_control (default: %(default)s)")

    parser.add_argument("--SJ_num_thres", type = int, default = 2,
                        help = "extract splicing junctions whose supporting numbers are equal or more than this value \
                        at least one sample in the cohort (default: %(default)s)")

    parser.add_argument("--keep_annotated", default = False, action = 'store_true',
                        help = "do not remove annotated splicing junctions")

    parser.add_argument("--IR_num_thres", type = int, default = 3,
                        help = "extract intron retentions whose supporting numbers are equal or more than this value \
                        and the ratio is equal or more than IR_ratio_thres at least one sample in the cohort (default: %(default)s)")

    parser.add_argument("--IR_ratio_thres", type = int, default = 0.05,
                        help = "extract intron retentions whose ratios (Intron_Retention_Read_Count / Edge_Read_Count) \
                        is equal or more than this value and supporting numbers are equal or more than IR_num_thres \
                        at least one sample in the cohort (default: %(default)s)")

    parser.add_argument("--chimera_num_thres", type = int, default = 2,
                        help = "minimum required number of supporting junction read pairs (default: %(default)s)")

    parser.add_argument("--chimera_overhang_thres", type = int, default = 30,
                        help = "region size which have to be covered by aligned short reads (default: %(default)s)")

    parser.add_argument("--permutation_num", type = int, default = 100,
                        help = "the number of permutation for calculating false discovery rate")

    parser.add_argument("--alpha0", type = float, default = 1.0,
                        help = "the shape parameter of prior Gamma Distribution for inactive states")

    parser.add_argument("--beta0", type = float, default = 1.0,
                        help = "the rate parameter of prior Gamma Distribution for inactive states") 

    parser.add_argument("--alpha1", type = float, default = 1.0,
                        help = "the shape parameter of prior Gamma Distribution for active states") 

    parser.add_argument("--beta1", type = float, default = 0.01,
                        help = "the rate parameter of prior Gamma Distribution for active states") 

    parser.add_argument("--log_BF_thres", type = float, default = 3.0,
                        help = "the threshould of logaraithm of Bayes Factor (default: %(default)s)")

    parser.add_argument("--effect_size_thres", type = float, default = 3.0,
                        help = "the thresould of effect size estimator used for simple edge pruning (default: %(default)s")

    parser.add_argument("--debug", default = False, action = 'store_true', help = "keep intermediate files")
    
    
    return parser
 




#! /usr/bin/env python

try:
    import cPickle as pickle
except:
    import pickle


import random
from collections import namedtuple
from network import Network
from sav import Sav

Link_info_mut = namedtuple("Link_info_mut", ("Mutation_Key", "Motif_Pos", "Mutation_Type", "Is_Canonical", "Splicing_Key", "Splicing_Class", "Is_Inframe"))
Link_info_sv = namedtuple("Link_info_sv", ("SV_Key", "SV_Type", "Splicing_Key", "Splicing_Class", "Is_Inframe"))


def get_mut_sample_info(mut_info, mut2sample):
    mut_infos = mut_info.split(',')
    tchr, tstart, tend, tref, talt = mut_infos[0], mut_infos[1], mut_infos[1], mut_infos[2], mut_infos[3]
    # deletion
    if len(tref) > 1:
        tref = tref[1:]
        talt = "-"
        tstart = str(int(tstart) + 1)
        tend = str(int(tstart) + len(tref) - 1)
    if len(talt) > 1:
        tref = "-"
        talt = talt[1:]

    mut = '\t'.join([tchr, tstart, tend, tref, talt])
    return(mut2sample[mut])


def create_network_list(merged_candidate_link_list, network_pickles_file, mut2sample_file, sample_list, weight_vector, sv_mode = False):

    # may need to fix when savnet is modified to accept vcf format file
    mut2sample = {}
    if sv_mode == False:
        with open(mut2sample_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                # mut = '\t'.join(F[0:5])
                mut = ','.join([F[0], F[1], F[3], F[4]])
                mut2sample[mut] = [int(x) - 1 for x in F[8].split(',')]
    else:
        with open(mut2sample_file, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                mut = ','.join(F[0:7])
                mut2sample[mut] = [int(x) - 1 for x in F[7].split(',')]


    out_s = open(network_pickles_file, 'wb')
    with open(merged_candidate_link_list, 'r') as hin:

        header2ind = {}
        header = hin.readline().rstrip('\n').split('\t')
        for i in range(len(header)):
            header2ind[header[i]] = i


        temp_gene = ""
        temp_mutation_status = {}
        temp_splicing_counts = []
        temp_link2info = {}
        
        temp_mut_id = 0
        temp_mutation_key2id = {}
        temp_splicing_id = 0 
        temp_splicing_key2id = {}

        for line in hin:

            F = line.rstrip('\n').split('\t')

            if F[header2ind["Gene_Symbol"]] != temp_gene:
                if temp_gene != "":
                    # flush the result
                    network = Network(temp_gene, temp_mutation_status, temp_splicing_counts, temp_link2info, sample_list, weight_vector)    
                    pickle.dump(Network(temp_gene, temp_mutation_status, temp_splicing_counts, temp_link2info, sample_list, weight_vector), out_s)

                temp_gene = F[header2ind["Gene_Symbol"]]
                temp_mutation_status = {}
                temp_splicing_counts = []
                temp_link2info = {}

                temp_mut_id = 0
                temp_mutation_key2id = {}
                temp_splicing_id = 0
                temp_splicing_key2id = {}
    
            # get mutation id and splicing id
            mutation_key = F[header2ind["Mutation_Key"]] if sv_mode == False else F[header2ind["SV_Key"]]
            if mutation_key not in temp_mutation_key2id:
                temp_mutation_key2id[mutation_key] = temp_mut_id
                # temp_mutation_status[temp_mut_id] =  get_mut_sample_info(mutation_key, mut2sample) if sv_mode == False else mut2sample[mutation_key]
                temp_mutation_status[temp_mut_id] = mut2sample[mutation_key]
                temp_mut_id = temp_mut_id + 1

            splicing_key = F[header2ind["Splicing_Key"]]
            if splicing_key not in temp_splicing_key2id:
                temp_splicing_key2id[splicing_key] = temp_splicing_id
                temp_splicing_counts.append([int(x) for x in F[header2ind["Read_Counts"]].split(',')])
                temp_splicing_id = temp_splicing_id + 1


            link = (temp_mutation_key2id[mutation_key], temp_splicing_key2id[splicing_key])

            ###
            if sv_mode == False:
                # This procedure is for mutation. Procedure for SV is also required
                mutation_key = F[header2ind["Mutation_Key"]]
                motif_pos = F[header2ind["Motif_Pos"]]
                mutation_type = F[header2ind["Mutation_Type"]]
                is_canonical = F[header2ind["Is_Canonical"]]
                splicing_class = F[header2ind["Splicing_Class"]]
                is_inframe = F[header2ind["Is_Inframe"]]

                temp_link2info[link] = Link_info_mut(mutation_key, motif_pos, mutation_type, is_canonical, splicing_key, splicing_class, is_inframe)
            else:
                mutation_key = F[header2ind["SV_Key"]]
                sv_type = F[header2ind["SV_Type"]]
                splicing_class = F[header2ind["Splicing_Class"]]
                is_inframe = F[header2ind["Is_Inframe"]]

                temp_link2info[link] = Link_info_sv(mutation_key, sv_type, splicing_key, splicing_class, is_inframe)
            ###


            ###
            # Is this necessary?
            """
            # consider the case where one mutation is both disrupting and creating splicing motifs
            if temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key] in temp_ids2link_info:
                # disrupting annotations are preferentially added to link info
                if F[header2ind["Mutation_Type"]] in ["splicing donor disruption", "splicing acceptor disruption"]:
                    temp_ids2link_info[temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key]] = link_info
                elif F[header2ind["Mutation_Type"]] in ["splicing donor creation", "splicing acceptor creation"] and \
                    "splicing branchpoint disruption" in temp_ids2link_info[temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key]]:
                    temp_ids2link_info[temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key]] = link_info
                # prefer canonical splicing branchpoint disruption rather than non-canonical splicing branchpoint disruption
                elif F[header2ind["Mutation_Type"]] == "splicing branchpoint disruption" and \
                    "splicing branchpoint disruption" in temp_ids2link_info[temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key]] and \
                    F[header2ind["Is_Canonical"]] == "canonical":
                    temp_ids2link_info[temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key]] = link_info
            else:
                temp_ids2link_info[temp_mut2id[mut] + '\t' + temp_splicing2id[splicing_key]] = link_info
            """
            ###


    # flush the result
    if temp_gene != "":
        pickle.dump(Network(temp_gene, temp_mutation_status, temp_splicing_counts, temp_link2info, sample_list, weight_vector), out_s)

    out_s.close()



def extract_sav_list(network_pickles_file, effect_size_thres, active_zero_filter_prob, inactive_nonzero_filter_prob, log_BF_thres, sample_num_thres,
                     alpha0, beta0, alpha1, beta1, permutation = False):

    sav_list = []
    seed = None 
    in_s = open(network_pickles_file, 'rb')
    while True:
        try:
            network = pickle.load(in_s)

            if permutation == True:
    
                # Set the seed
                if seed is None:
                    seed = range(network.sample_num)
                    no_overlap = 0
                    while no_overlap == 0:
                        random.shuffle(seed)
                        overlap_num = sum([seed[i] == i for i in range(network.sample_num)])
                        if overlap_num == 0: no_overlap = 1 
               
                network.execute_permutation(seed)

            network.prune_link_vector(effect_size_thres, active_zero_filter_prob, inactive_nonzero_filter_prob)
            network.cluster_link_vector()
            network.get_averaged_bayes_factors(alpha0, beta0, alpha1, beta1)
            
            for sav in network.export_to_savs(log_BF_thres, sample_num_thres):
                sav_list.append(sav)
        
        except EOFError:
            break
    
    return sav_list


def add_qvalue_to_sav_list(sav_list_target, sav_lists_permutation):

    permutation_num = len(sav_lists_permutation)
    # print permutation_num

    logBF_values_target = [x.score for x in sav_list_target]

    logBF_values_null = []
    for i in range(permutation_num):
        logBF_values_null = logBF_values_null + [x.score for x in sav_lists_permutation[i]]

    logBF2FPR = {}
    for logBF in logBF_values_target:
        null_num_est = float(len([x for x in logBF_values_null if x >= logBF])) / permutation_num
        nonnull_num = float(len([x for x in logBF_values_target if x >= logBF]))
        pFPR = null_num_est / nonnull_num if nonnull_num > 0.0 else 0.0
        logBF2FPR[logBF] = pFPR
    
    logBF2qvalue = {}
    temp_min_FPR = float("inf")
    for logBF in sorted(logBF2FPR, key = float):
        if logBF2FPR[logBF] <= temp_min_FPR:
            temp_min_FPR = logBF2FPR[logBF]
        logBF2qvalue[logBF] = temp_min_FPR

    for sav in sav_list_target:
        sav.set_fdr(logBF2qvalue[sav.score])


def execute_network_analysis(network_pickles_file, savnet_target_result_file, savnet_permutation_result_file, 
                             effect_size_thres, log_BF_thres, alpha0, beta0, alpha1, beta1, permutation_num):

    sav_list_target = extract_sav_list(network_pickles_file, effect_size_thres, log_BF_thres, alpha0, beta0, alpha1, beta1, permutation = False)
 
    sav_lists_permutation = []
    for i in range(permutation_num):
        temp_sav_list = extract_sav_list(network_pickles_file, effect_size_thres, log_BF_thres, alpha0, beta0, alpha1, beta1, permutation = True)
        sav_lists_permutation.append(temp_sav_list)

    add_qvalue_to_sav_list(sav_list_target, sav_lists_permutation)

    with open(savnet_target_result_file, 'w') as hout:
        print >> hout, Sav.print_header_mut
        for sav in sav_list_target:
            print >> hout, sav.print_records()


if __name__ == "__main__":

    import sys

    sample_list = ["A427", "A549", "ABC-1", "H1299", "H1437", "H1648", "H1650", "H1703", "H1819", 
                   "H1975", "H2126", "H2228", "H2347", "H322", "II-18", "LC2_ad", "PC-14", "PC-3", 
                   "PC-7", "PC-9", "RERF-LC-Ad1", "RERF-LC-Ad2", "RERF-LC-KJ", "RERF-LC-MS", "RERF-LC-OK", "VMRC-LCD"]

    weight_vector = [5.2935, 2.5843, 4.6718, 6.7032, 6.3449, 4.8819, 3.2902, 11.6925, 9.7603, 4.4712, 5.8023, 7.3304, 6.6022,
                     6.8118, 8.0286, 5.5164, 6.7205, 6.3547, 6.5036, 4.2754, 6.989, 5.63, 7.514, 6.6156, 4.1002, 6.0268]

    create_network_list(sys.argv[1], sys.argv[2], sys.argv[3], sample_list, weight_vector)

    execute_network_analysis(sys.argv[2], "test.savnet.result.txt", None, 3.0, 3.0, 1.0, 1.0, 1.0, 0.01, 100)






#! /usr/bin/env python

import math, sys, os
from subprocess import Popen, PIPE

from logger import get_logger
logger = get_logger(__name__)


def soft_max(x):
    x_max = max(x)
    return(x_max + math.log(sum([math.exp(y - x_max) for y in x]))) 


def median(numbers):
    if len(numbers) == 0:
        logger.warning("Vector of zero length was put to median function. Return 0")
        return 0
    # return (sorted(numbers)[int(round((len(numbers) - 1) / 2.0))] + sorted(numbers)[int(round((len(numbers) - 1) // 2.0))]) / 2.0
    return sorted(numbers)[int(round((len(numbers) - 1) / 2.0))]


def quartile(numbers, prob):
    if len(numbers) == 0:
        logger.warning("Vector of zero length was put to quartile function. Return 0")
        return 0
    return sorted(numbers)[int((len(numbers) - 1) * prob)]


def generate_configurations(dim):

    conf = [[0], [1]]

    for k in range(dim - 1):

        new_conf = []
        for elm in conf:
            new_conf.append(elm + [0])
            new_conf.append(elm + [1])
            conf = new_conf

    return conf


def is_tool(executable):

    proc = Popen(["which", executable], stdout = PIPE, stderr = PIPE)
    out, err = proc.communicate()
    if proc.returncode == 1:
        logger.error("Executable does not exist: " + executable)
        sys.exit(1) 

    return True





#! /usr/bin/env python

import sys, subprocess, os
import preprocess, analysis_network, sample_conf
from sav import Sav
from utils import is_tool

from logger import get_logger
logger = get_logger(__name__)


def savnet_main(args):

    if args.grc == True:
        logger.warning("--grc argument is deprecated and ignored.")


    ##########
    # read sample conf
    sconf = sample_conf.Sample_conf()
    sconf.parse_file(args.sample_list_file, args.sv)

    ##########
    # check if the executables exist
    is_tool("bedtools")
    is_tool("tabix")
    is_tool("bgzip")
    if len(sconf.SJ_files) > 0: is_tool("junc_utils")
    if len(sconf.IR_files) > 0: is_tool("intron_retention_utils")
    if len(sconf.chimera_files) > 0: is_tool("chimera_utils")
    
    ##########
    output_prefix_dir = os.path.dirname(args.output_prefix)
    if output_prefix_dir != "" and not os.path.exists(output_prefix_dir):
       os.makedirs(output_prefix_dir)

    ##########
    logger.info("Merging mutation data.")
    if args.sv == False:
        preprocess.merge_mut2(sconf.mut_files, args.output_prefix + ".mut_merged.vcf", args.reference)
    else:
        preprocess.merge_sv(sconf.sv_files, args.output_prefix + ".sv_merged.txt")

    ##########
    # splicing_junction
    logger.info("Merging splicing junction data.")
    preprocess.merge_SJ2(sconf.SJ_files, args.output_prefix + ".SJ_merged.txt", args.SJ_pooled_control_file, args.SJ_num_thres, args.keep_annotated)

    logger.info("Adding annotation to splicing junction data.")
    annotate_commands = ["junc_utils", "annotate", args.output_prefix + ".SJ_merged.txt", args.output_prefix + ".SJ_merged.annot.txt",
                         "--genome_id", args.genome_id]
    # if args.grc: annotate_commands.append("--grc")
    subprocess.call(annotate_commands)

    logger.info("Checking association betweeen mutation and splicing junction data.")
    if args.sv == False:
        associate_commands = ["junc_utils", "associate", args.output_prefix + ".SJ_merged.annot.txt", args.output_prefix + ".mut_merged.vcf",
                              args.output_prefix + ".SJ_merged.associate.txt",
                              "--donor_size", args.donor_size, "--acceptor_size", args.acceptor_size,
                              "--genome_id", args.genome_id]
        # if args.branchpoint: associate_commands.append("--branchpoint")
        # if args.grc: associate_commands.append("--grc")

    else:
        associate_commands = ["junc_utils", "associate", args.output_prefix + ".SJ_merged.annot.txt", args.output_prefix + ".sv_merged.txt",
                              args.output_prefix + ".SJ_merged.associate.txt", "--sv"]

    subprocess.check_call(associate_commands)
    ##########

    ##########
    # intron_retention
    logger.info("Merging intron retention data.")
    preprocess.merge_intron_retention(sconf.IR_files, args.output_prefix + ".IR_merged.txt", 
                                 args.IR_pooled_control_file, args.IR_ratio_thres, args.IR_num_thres)

    logger.info("Checking association betweeen mutation and intron retention data.")
    if args.sv == False:
        associate_commands = ["intron_retention_utils", "associate", args.output_prefix + ".IR_merged.txt",
                              args.output_prefix + ".mut_merged.vcf", args.output_prefix + ".IR_merged.associate.txt",
                              "--donor_size", args.donor_size, "--acceptor_size", args.acceptor_size]
    else:
        associate_commands = ["intron_retention_utils", "associate", args.output_prefix + ".IR_merged.txt",
                              args.output_prefix + ".sv_merged.txt", args.output_prefix + ".IR_merged.associate.txt", "--sv"]

    subprocess.check_call(associate_commands)
    #########

    #########
    # chimera
    if args.sv:
        logger.info("Merging chimeric junction data.")
        preprocess.merge_chimera(sconf.chimera_files, args.output_prefix + ".chimera_merged.txt", 
                            args.chimera_pooled_control_file, args.chimera_num_thres, args.chimera_overhang_thres)

        logger.info("Checking association betweeen mutation and chimeric junction data.")
        associate_commands = ["chimera_utils", "associate", args.output_prefix + ".chimera_merged.txt",
                              args.output_prefix + ".sv_merged.txt", args.output_prefix + ".chimera_merged.associate.txt", "--genome_id", args.genome_id]
        # if args.grc: associate_commands.append("--grc")

        subprocess.check_call(associate_commands)
    ##########

    ##########
    # organize association
    if args.sv == False:
        logger.info("Organizing splicing association information.")
        preprocess.merge_SJ_IR_files(args.output_prefix + ".SJ_merged.associate.txt", 
                                     args.output_prefix + ".IR_merged.associate.txt",
                                     args.output_prefix + ".splicing.associate.txt")

        logger.info("Creating pickles of splicing association network instances.")
        analysis_network.create_network_list(args.output_prefix + ".splicing.associate.txt",
                                             args.output_prefix + ".splicing_mutation.network.pickles",
                                             args.output_prefix + ".mut_merged.vcf",
                                             sconf.sample_names, sconf.weights)

    else:
        logger.info("Organizing splicing association information.")
        preprocess.merge_SJ_IR_chimera_files_sv(args.output_prefix + ".SJ_merged.associate.txt",
                                                args.output_prefix + ".IR_merged.associate.txt",
                                                args.output_prefix + ".chimera_merged.associate.txt",
                                                args.output_prefix + ".splicing.associate.txt")

        logger.info("Creating pickles of splicing association network instances.")
        analysis_network.create_network_list(args.output_prefix + ".splicing.associate.txt", 
                                             args.output_prefix + ".splicing_mutation.network.pickles",
                                             args.output_prefix + ".sv_merged.txt",
                                             sconf.sample_names, sconf.weights, sv_mode = True)


    logger.info("Extracting splicing associated variants.")
    sav_list_target = analysis_network.extract_sav_list(args.output_prefix + ".splicing_mutation.network.pickles", 
                                                        args.effect_size_thres, 0.5, 0.5, args.log_BF_thres, 1, 
                                                        args.alpha0, args.beta0, args.alpha1, args.beta1, permutation = False)

    logger.info("Extracting of splicing associated variants on permutation pairs to estimate false positive ratios.")
    sav_lists_permutation = []
    for i in range(args.permutation_num):
        temp_sav_list = analysis_network.extract_sav_list(args.output_prefix + ".splicing_mutation.network.pickles", 
                                                          args.effect_size_thres, 0.5, 0.5, args.log_BF_thres, 1,
                                                          args.alpha0, args.beta0, args.alpha1, args.beta1, permutation = True)
        sav_lists_permutation.append(temp_sav_list)

    logger.info("Adding Q-values to splicing associated variants.")
    analysis_network.add_qvalue_to_sav_list(sav_list_target, sav_lists_permutation)

    logger.info("Generating final outputs.")
    with open(args.output_prefix + ".savnet.result.txt", 'w') as hout:
        if args.sv == False:
            print >> hout, Sav.print_header_mut 
        else:
            print >> hout, Sav.print_header_sv
        for sav in sav_list_target:
            print >> hout, '\n'.join(sav.print_records(sv_mode = args.sv, with_fdr = True))

    with open(args.output_prefix + ".splicing_mutation.count_summary.anno.perm_all.txt", 'w') as hout:
        if args.sv == False:
            print >> hout, "Permutation_Num" + '\t' + Sav.print_header_mut
        else:
            print >> hout, "Permutation_Num" + '\t' + Sav.print_header_sv

        for i in range(len(sav_lists_permutation)):
            for sav in sav_lists_permutation[i]:
                print >> hout, '\n'.join([str(i) + '\t' + x for x in sav.print_records(sv_mode = args.sv, with_fdr = False)])

    if args.debug == False:

        subprocess.call(["rm", "-rf", args.output_prefix + ".mut_merged.vcf"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".sv_merged.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".SJ_merged.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".SJ_merged.annot.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".SJ_merged.associate.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".IR_merged.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".IR_merged.associate.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".chimera_merged.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".chimera_merged.associate.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".splicing.associate.txt"])
        subprocess.call(["rm", "-rf", args.output_prefix + ".splicing_mutation.network.pickles"]) 

